/////////////////////////////////////////////////////////////////////////////////////////
//
// TheUno DS18B20 Thermometer
//
// --------------------------------------------------------------------------------------
//
// Filename:      main.c
// Version:       1.2
// Date:          31/07/2012
// Author:        Joel Guittet - http://myfreescalewebpage.free.fr
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Description
//
// This project shows how to simply use a DS18B20 temperature sensor on TheUno.
// The temperature is periodically read and the temperature is displayed on two
// classical digits.
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Revisions
//
// Version	| Author		| Description
// --------------------------------------------------------------------------------------
// 1.0      | Joel Guittet  | Initial Release
// 1.1      | Joel Guittet  | Coding improvements
// 1.1      | Joel Guittet  | Modifications to use "typedef.h"
// 1.1      | Joel Guittet  | Integration of 1-Wire library V1.2
// 1.2      | Joel Guittet  | Coding improvements
//
/////////////////////////////////////////////////////////////////////////////////////////


//---------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------

#include <hidef.h>
#include "derivative.h"
#include "ds18b20.h"


//---------------------------------------------------------------------------------------
// Prototypes
//---------------------------------------------------------------------------------------

static void InitCPU(void);
static void InitMCG(void);
static void InitGPIO(void);
static void DisplayTemperature(UINT8 u8Value);


//---------------------------------------------------------------------------------------
// Name:        main
// Param:		-
// Return:      -
//
// Description:	Main function, never return
//---------------------------------------------------------------------------------------
void main(void)
{
	UINT16 u16Index, u16Temperature;
	t_OneWireResetResult xDS18B20Status = NO_DEVICE_DETECTED;
		
	/* Initializations */
	InitCPU();
	
	/* Initialize the DS18B20 sensor, temperature resolution set to 9bits */
	xDS18B20Status = DS18B20_Init(9);

	EnableInterrupts;

	/* Main loop */
	for(;;)
	{
		/* Initiate a new temperature conversion (take up to 94ms) */
		xDS18B20Status = DS18B20_ConvertT();
		
		/* Make a small delay */
		for (u16Index = 0; u16Index < 65535; u16Index++);
		
		/* Now read the temperature and display the value on the digits */
		xDS18B20Status = DS18B20_ReadT(&u16Temperature);
		
		DisplayTemperature((UINT8)(u16Temperature >> 4));
		
		/* Make a small delay */
		for (u16Index = 0; u16Index < 65535; u16Index++);

		__RESET_WATCHDOG();
	}
}


//---------------------------------------------------------------------------------------
// Name:        InitCPU
// Param:		-
// Return:      -
//
// Description:	Initialize CPU
//---------------------------------------------------------------------------------------
static void InitCPU(void)
{
	InitMCG();
	InitGPIO();
}


//---------------------------------------------------------------------------------------
// Name:        InitMCG
// Param:    	-
// Return:      -
//
// Description:	MCG configuration: Crystal=12MHz, Fbus = 18MHz
//---------------------------------------------------------------------------------------
static void InitMCG(void)
{
	MCGC2 = 0x36;
	while(MCGSC_OSCINIT != 1);
	MCGC1 = 0xB8;
	while(MCGSC_IREFST != 0);
	while(MCGSC_CLKST != 0b10);  
	MCGC1 = 0x98;
	MCGC3 = 0x46;
	while(MCGSC_PLLST != 1);
	while(MCGSC_LOCK != 1);
	MCGC1 = 0x18;
	while(MCGSC_CLKST != 0b11);
}


//---------------------------------------------------------------------------------------
// Name:        InitGPIO
// Param:    	-
// Return:      -
//
// Description:	IO ports initialization
//---------------------------------------------------------------------------------------
static void InitGPIO(void)
{
	PTDDD_PTDDD3 = 1;
	PTDDD_PTDDD4 = 1;
	PTADD_PTADD5 = 1;
	PTADD_PTADD6 = 1;
	PTEDD_PTEDD5 = 1;
	PTADD_PTADD7 = 1;
	PTBDD_PTBDD0 = 1;
	PTEDD_PTEDD6 = 1;
	PTEDD_PTEDD7 = 1;
	PTADD_PTADD1 = 1;
	PTADD_PTADD3 = 1;
	PTADD_PTADD4 = 1;
	PTBDD_PTBDD1 = 1;
	PTADD_PTADD2 = 1;
	
	PTDD_PTDD3 = 1;
	PTDD_PTDD4 = 1;
	PTAD_PTAD5 = 1;
	PTAD_PTAD6 = 1;
	PTED_PTED5 = 1;
	PTAD_PTAD7 = 1;
	PTBD_PTBD0 = 1;
	PTED_PTED6 = 1;
	PTED_PTED7 = 1;
	PTAD_PTAD1 = 1;
	PTAD_PTAD3 = 1;
	PTAD_PTAD4 = 1;
	PTBD_PTBD1 = 1;
	PTAD_PTAD2 = 1;
}


//---------------------------------------------------------------------------------------
// Name:        DisplayTemperature
// Param:		Value: temperature value to display on the digits
// Return:      -
//
// Description:	This function displays the temperature on the digits
//---------------------------------------------------------------------------------------
static void DisplayTemperature(UINT8 u8Value)
{
	UINT8 u8Digit1, u8Digit2;
	
	u8Digit1 = u8Value / 10;
	u8Digit2 = u8Value % 10;
	
	/* Take care, digits are common anode digits, which mean that the LEDs are ON if the output pins on the microcontroller are low */
	
	/* Digit 1 */
	PTDD_PTDD3 = (u8Digit1 == 1) || (u8Digit1 == 4);																				/* a */
	PTDD_PTDD4 = (u8Digit1 == 5) || (u8Digit1 == 6);																				/* b */
	PTAD_PTAD5 = (u8Digit1 == 2);																									/* c */
	PTAD_PTAD6 = (u8Digit1 == 1) || (u8Digit1 == 4) || (u8Digit1 == 7);																/* d */
	PTED_PTED5 = (u8Digit1 == 1) || (u8Digit1 == 3) || (u8Digit1 == 4) || (u8Digit1 == 5) || (u8Digit1 == 7) || (u8Digit1 == 9);	/* e */
	PTAD_PTAD7 = (u8Digit1 == 1) || (u8Digit1 == 2) || (u8Digit1 == 3) || (u8Digit1 == 7);											/* f */
	PTBD_PTBD0 = (u8Digit1 == 1) || (u8Digit1 == 7) || (u8Digit1 == 0);																/* g */
	
	/* Digit 2 */
	PTED_PTED6 = (u8Digit2 == 1) || (u8Digit2 == 4);																				/* a */
	PTED_PTED7 = (u8Digit2 == 5) || (u8Digit2 == 6);																				/* b */
	PTAD_PTAD1 = (u8Digit2 == 2);																									/* c */
	PTAD_PTAD3 = (u8Digit2 == 1) || (u8Digit2 == 4) || (u8Digit2 == 7);																/* d */
	PTAD_PTAD4 = (u8Digit2 == 1) || (u8Digit2 == 3) || (u8Digit2 == 4) || (u8Digit2 == 5) || (u8Digit2 == 7) || (u8Digit2 == 9);	/* e */
	PTBD_PTBD1 = (u8Digit2 == 1) || (u8Digit2 == 2) || (u8Digit2 == 3) || (u8Digit2 == 7);											/* f */
	PTAD_PTAD2 = (u8Digit2 == 1) || (u8Digit2 == 7) || (u8Digit2 == 0);																/* g */
}
